
# q-expansion of newform 112.2.w.b, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 112
weight = 2
poly_data = [1, 0, -1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[15, [1, 0, 0, 0]], [85, [0, 0, 0, 1]], [17, [-1, 0, 1, 0]]]
aps_data = [[1, 0, 0, 1], [1, 0, -1, 1], [-1, -1, 2, -1], [0, 2, 0, -3], [-3, 2, 1, 1], [3, 2, -2, -3], [0, -2, -3, -2], [2, -2, 1, 3], [2, 1, 2, 0], [-1, 2, -2, 1], [0, -1, -2, -1], [1, 1, -6, 5], [-4, 0, 8, -2], [1, 4, 4, 1], [-6, 1, 6, -2], [3, -7, 4, 4], [-9, 4, 5, 5], [-1, 1, 4, 3], [1, -4, -5, 5], [-2, 0, 4, -4], [6, 2, -3, -2], [8, 5, -8, -10], [3, -4, 4, -3], [-3, 0, -3, 0], [4, -8, 0, 4], [5, 7, -12, -12], [-2, -3, -2, 0], [2, 2, -1, -1], [-7, 1, 8, -8], [2, -4, 0, 2], [6, 4, 0, -2], [-4, 4, -3, -7], [18, 2, -9, -2], [-1, -8, -8, -1], [-3, -3, -4, 7], [12, 1, -6, -1], [-1, -13, -12, 12], [-2, 2, 7, 5], [-8, 0, 16, 8], [-3, 3, 2, -1], [7, 4, -3, 3], [-3, 12, 12, -3], [0, -5, 0, 10], [0, 0, 23, 0], [-3, -10, -10, -3], [-8, -5, 4, 5], [-13, -8, 8, 13], [-4, 16, 0, -8], [-5, 10, 15, -15], [5, 5, 8, -13], [-3, 6, -3, 0], [-12, 4, 0, -2], [0, -4, -7, -4], [7, -18, -18, 7], [9, 6, -9, -12], [-12, 15, 6, -15], [-5, 9, 14, -14], [0, -7, 0, 14], [-9, 9, 0, 0], [-4, 0, 8, -6], [1, -12, 11, 11], [3, -8, -8, 3], [9, 0, 0, -9], [4, 3, -2, -3], [-9, -6, -9, 0], [9, -9, -10, -1], [-14, -14, 13, 1], [-20, 16, 0, -8], [-17, 6, 11, 11], [1, 14, -14, -1], [0, 12, 7, 12], [-2, -1, -2, 0], [0, -7, 6, -7], [1, 1, -2, 1], [-19, 14, 14, -19], [20, 3, -20, -6], [1, 15, -16, -16], [-1, 1, 14, 13], [15, 0, -15, 0], [-10, 0, 5, 0], [-19, 0, 0, 19], [-5, 10, 10, -5], [0, -5, -22, -5], [-2, -36, 0, 18], [-2, 11, -2, 0], [-8, -8, 9, -1], [-6, -20, 0, 10], [-3, 24, -3, 0], [15, 10, -10, -15], [16, -16, 0, 8], [4, -4, -15, -11], [0, -9, 0, -9], [20, -5, -10, 5], [21, -18, -18, 21], [-6, 6, 15, 9], [-8, 0, 16, 18], [-11, 11, 8, -3], [-34, 14, 17, -14], [-24, -24, 25, -1], [11, -11, 2, 13], [-15, -22, 22, 15], [5, 3, -2, 2], [17, -8, -25, 25], [7, -18, 7, 0], [-7, -10, 17, 17], [0, -14, 11, -14], [11, 8, 8, 11], [1, 6, -1, -12], [28, -5, -14, 5], [0, 0, 0, -10], [2, -11, -2, 22], [13, -11, -2, -2], [-2, 0, 4, -4], [19, -12, -7, -7], [14, 0, -28, -8], [0, -8, 25, -8], [-1, -12, 12, 1], [40, 1, -20, -1], [17, -17, -22, -5], [13, 16, -16, -13], [-17, 7, 10, 10], [20, 24, 0, -12], [-17, -17, -22, 39], [1, 32, -33, -33], [-10, 10, 15, 5], [-9, -12, 12, 9], [25, 25, -2, -23], [-16, 9, 16, -18], [-10, 0, 20, 24], [-21, 21, -12, -33], [-5, 30, 35, -35], [4, 0, -8, 18], [6, -11, -6, 22], [-5, -10, -10, -5], [9, -2, 9, 0], [-4, 16, 0, -8], [-11, 7, 4, 4], [1, -10, -11, 11], [-17, 10, -10, 17], [-30, -8, 15, 8], [-3, 20, 20, -3], [3, 3, -10, 7], [-28, 7, 14, -7], [29, -24, -24, 29], [13, -7, -20, 20], [16, 0, -32, 10], [-31, 14, 14, -31], [2, 12, -1, -12], [14, 14, -25, 11], [24, 5, -24, -10], [-21, 21, 22, 1], [50, 0, 0, 0], [-5, 0, 0, 5], [-4, -39, -4, 0], [7, 10, -17, -17], [-10, 20, 0, -10], [-10, -1, -10, 0], [-5, -26, 5, 52], [4, 0, -8, 36], [-17, -11, 6, -6], [-22, 22, 3, -19], [-2, 0, 4, -20], [-14, 0, 28, 36], [-38, -38, 37, 1], [0, -8, 31, -8], [24, -1, -12, 1], [0, -9, 24, -9], [3, -3, 0, 0]]
